class ElementalPower(object):
    def __init__(self, element: str, attack: int, dmg: int, res: int):
        self.element = element
        self.attack = attack
        self.dmg = dmg
        self.res = res



class Slot(object):
    def __init__(self, content: str, quantity: int):
        self.content = content
        self.quantity = quantity


class Equipment(object):
    def __init__(
        self,
        weapon_slot: str = None,
        shield_slot: str = None,
        helmet_slot: str = None,
        body_armor_slot: str = None,
        leg_armor_slot: str = None,
        boots_slot: str = None,
        ring1_slot: str = None,
        ring2_slot: str = None,
        amulet_slot: str = None,
        artifact1_slot: str = None,
        artifact2_slot: str = None,
        artifact3_slot: str = None,
        consumable1_slot: str = None,
        consumable1_slot_quantity: int = 0,
        consumable2_slot: str = None,
        consumable2_slot_quantity: int = 0,
    ):
        self.weapon_slot = Slot(weapon_slot, 1)
        self.shield_slot = Slot(shield_slot, 1)
        self.helmet_slot = Slot(helmet_slot, 1)
        self.body_armor_slot = Slot(body_armor_slot, 1)
        self.leg_armor_slot = Slot(leg_armor_slot, 1)
        self.boots_slot = Slot(boots_slot, 1)
        self.ring_slots = [Slot(ring1_slot, 1), Slot(ring2_slot, 1)]
        self.amulet_slot = Slot(amulet_slot, 1)
        self.artifact_slots = [
            Slot(artifact1_slot, 1),
            Slot(artifact2_slot, 1),
            Slot(artifact3_slot, 1),
        ]
        self.consumable_slots = [
            Slot(consumable1_slot, consumable1_slot_quantity),
            Slot(consumable2_slot, consumable2_slot_quantity),
        ]


class Inventory(object):
    def __init__(self, data: list, inventory_max_items: int = 20):
        self.slots = [None for _ in range(inventory_max_items)]
        for slot_data in data:
            self.slots[slot_data["slot"]] = Slot(
                slot_data["code"], slot_data["quantity"]
            )
        self.max_items = inventory_max_items



import datetime
from .skins import Skin
from .skills import Skill
from .elemental_powers import ElementalPower
from .positions import Position
from .possesions import Equipment, Inventory
from ..client import Client


class Character(object):
    def __init__(self, client: Client, name: str):
        self._client = client
        self.name = name
        self.update_from_server()
        self.cooldown_expiration = None

    def update_from_server(self):
        data = self._client.get_character_data(self.name)
        self.update_from_data(data)

    def update_from_data(self, data: dict):
        self.skin = data["skin"]
        self.level = data["level"]
        self.xp = data["xp"]
        self.max_xp = data["max_xp"]
        self.total_xp = data["total_xp"]
        self.gold = data["gold"]
        self.speed = data["speed"]
        mining_skill = Skill(
            "mining", data["mining_level"], data["mining_xp"], data["mining_max_xp"]
        )
        woodcutting_skill = Skill(
            "woodcutting",
            data["woodcutting_level"],
            data["woodcutting_xp"],
            data["woodcutting_max_xp"],
        )
        fishing_skill = Skill(
            "fishing", data["fishing_level"], data["fishing_xp"], data["fishing_max_xp"]
        )
        weaponcrafting_skill = Skill(
            "weaponcrafting",
            data["weaponcrafting_level"],
            data["weaponcrafting_xp"],
            data["weaponcrafting_max_xp"],
        )
        gearcrafting_skill = Skill(
            "gearcrafting",
            data["gearcrafting_level"],
            data["gearcrafting_xp"],
            data["gearcrafting_max_xp"],
        )
        jewelrycrafting_skill = Skill(
            "jewelrycrafting",
            data["jewelrycrafting_level"],
            data["jewelrycrafting_xp"],
            data["jewelrycrafting_max_xp"],
        )
        cooking_skill = Skill(
            "cooking", data["cooking_level"], data["cooking_xp"], data["cooking_max_xp"]
        )
        self.skills = {
            "mining": mining_skill,
            "woodcutting": woodcutting_skill,
            "fishing": fishing_skill,
            "weaponcrafting": weaponcrafting_skill,
            "gearcrafting": gearcrafting_skill,
            "jewelrycrafting": jewelrycrafting_skill,
            "cooking": cooking_skill,
        }
        self.hp = data["hp"]
        self.haste = data["haste"]
        self.critical_strike = data["critical_strike"]
        self.stamina = data["stamina"]
        fire_power = ElementalPower(
            "fire", data["attack_fire"], data["dmg_fire"], data["res_fire"]
        )
        earth_power = ElementalPower(
            "earth", data["attack_earth"], data["dmg_earth"], data["res_earth"]
        )
        water_power = ElementalPower(
            "water", data["attack_water"], data["dmg_water"], data["res_water"]
        )
        air_power = ElementalPower(
            "air", data["attack_air"], data["dmg_air"], data["res_air"]
        )
        self.elemental_powers = {
            "fire": fire_power,
            "earth": earth_power,
            "water": water_power,
            "air": air_power,
        }
        self.position = Position(data["x"], data["y"])
        self.cooldown = data["cooldown"]
        self.cooldown_expiration = data["cooldown_expiration"]
        self.equipment = Equipment(
            data["weapon_slot"],
            data["shield_slot"],
            data["helmet_slot"],
            data["body_armor_slot"],
            data["leg_armor_slot"],
            data["boots_slot"],
            data["ring1_slot"],
            data["ring2_slot"],
            data["amulet_slot"],
            data["artifact1_slot"],
            data["artifact2_slot"],
            data["artifact3_slot"],
            data["consumable1_slot"],
            data["consumable1_slot_quantity"],
            data["consumable2_slot"],
            data["consumable2_slot_quantity"],
        )
        self.inventory_max_items = data["inventory_max_items"]
        self.inventory = Inventory(data["inventory"], self.inventory_max_items)

    def move(self, position: Position = None, x: int = None, y: int = None):
        if position is not None:
            x = position.x
            y = position.y
        elif x is None or y is None:
            raise ValueError("Either position or x and y must be provided.")
        data = self._client.move_character(self.name, x, y)
        self.update_from_data(data["character"])
        self.cooldown_expiration = datetime.datetime.now() + datetime.timedelta(
            seconds=data["cooldown"]["remainingSeconds"]
        )

    def __str__(self):
        return f"<Character: {self.name} @ {self.position} - Level {self.level} - {self.hp} HP - {self.gold} gold>"



from typing import Union
import requests
import logging


class Client(object):
    """Client for interacting with the Artifacts MMO API."""

    def __init__(self, host: str = "https://api.artifactsmmo.com", token: str = None):
        """
        :param host: The host to connect to.
        :param token: The token to use for authentication.
        """
        self.host = host
        self.token = token

    def get_server_status(self) -> dict:
        """Get the server status."""
        try:
            response = requests.get(self.host)
            response.raise_for_status()
            return response.json()["data"]
        except requests.exceptions.RequestException as e:
            logging.error(f"Failed to get server status: {e}")
            return {
                "status": "error",
                "error": str(e),
            }

    def get_character_data(self, name: str) -> Union[dict, None]:
        """Get a character by name."""
        try:
            response = requests.get(f"{self.host}/characters/{name}")
            response.raise_for_status()
            return response.json()["data"]
        except requests.exceptions.RequestException as e:
            if response.status_code == 404:
                logging.error(f"Character '{name}' not found.")
                return None
            else:
                logging.error(f"Failed to get character '{name}' data: {e}")
                return None

    def move_character(self, name: str, x: int, y: int) -> dict:
        """Move a character to a new location."""
        try:
            response = requests.post(
                f"{self.host}/my/{name}/action/move",
                headers={"Authorization": f"Bearer {self.token}"},
                json={"x": x, "y": y},
            )
            response.raise_for_status()
            return response.json()["data"]
        except requests.exceptions.RequestException as e:
            if response.status_code == 404:
                logging.error(f"Map ({x}, {y}) not found.")
            elif response.status_code == 486:
                logging.error(
                    f"Character '{name}' is locked. Action is already in progress."
                )
            elif response.status_code == 490:
                logging.error(f"Character '{name}' already at destination ({x}, {y}).")
            elif response.status_code == 498:
                logging.error(f"Character '{name}' not found.")
            elif response.status_code == 499:
                logging.error(f"Character '{name}' in cooldown.")
            else:
                logging.error(f"Failed to move character '{name}' to ({x}, {y}): {e}")
                return None
